@@ -10,7 +10,8 @@ from __future__ import absolute_import
 import logging
 import time
 
-# Import Salt lobs
+# Import Salt libs
+import salt.config
 from salt.ext import six
 from salt.payload import Serial
 from salt.utils.odict import OrderedDict
@@ -66,12 +67,17 @@ class Cache(object):
     Key name is a string identifier of a data container (like a file inside a
     directory) which will hold the data.
     '''
-    def __init__(self, opts, **kwargs):
+    def __init__(self, opts, cachedir=None, **kwargs):
         self.opts = opts
-        self.driver = opts['cache']
+        if cachedir is None:
+            self.cachedir = opts.get('cachedir', salt.syspaths.CACHE_DIR)
+        else:
+            self.cachedir = cachedir
+        self.driver = opts.get('cache', salt.config.DEFAULT_MASTER_OPTS)
         self.serial = Serial(opts)
         self._modules = None
         self._kwargs = kwargs
+        self._kwargs['cachedir'] = self.cachedir
 
     def __lazy_init(self):
         self._modules = salt.loader.cache(self.opts, self.serial)
@@ -218,7 +224,7 @@ class Cache(object):
         fun = '{0}.flush'.format(self.driver)
         return self.modules[fun](bank, key=key, **self._kwargs)
 
-    def ls(self, bank):
+    def list(self, bank):
         '''
         Lists entries stored in the specified bank.
 
@@ -234,11 +240,9 @@ class Cache(object):
             Raises an exception if cache driver detected an error accessing data
             in the cache backend (auth, permissions, etc).
         '''
-        fun = '{0}.ls'.format(self.driver)
+        fun = '{0}.list'.format(self.driver)
         return self.modules[fun](bank, **self._kwargs)
 
-    list = ls
-
     def contains(self, bank, key=None):
         '''
         Checks if the specified bank contains the specified key.
