@@ -23,7 +23,7 @@ import salt.utils.atomicfile
 
 log = logging.getLogger(__name__)
 
-__func_alias__ = {'list': 'ls'}
+__func_alias__ = {'list_': 'list'}
 
 
 def __cachedir(kwargs=None):
@@ -143,7 +143,7 @@ def flush(bank, key=None, cachedir=None):
     return True
 
 
-def ls(bank, cachedir):
+def list_(bank, cachedir):
     '''
     Return an iterable object containing all entries stored in the specified bank.
     '''
