@@ -359,29 +359,19 @@ class SyncClientMixin(object):
             # packed into the top level object. The plan is to move away from
             # that since the caller knows what is an arg vs a kwarg, but while
             # we make the transition we will load "kwargs" using format_call if
-            # there are no kwargs in the low object passed in
-            f_call = None
-            if 'arg' not in low:
+            # there are no kwargs in the low object passed in.
+
+            if 'arg' in low and 'kwarg' in low:
+                args = low['arg']
+                kwargs = low['kwarg']
+            else:
                 f_call = salt.utils.format_call(
                     self.functions[fun],
                     low,
                     expected_extra_kws=CLIENT_INTERNAL_KEYWORDS
                 )
                 args = f_call.get('args', ())
-            else:
-                args = low['arg']
-
-            if 'kwarg' not in low:
-                log.critical(
-                    'kwargs must be passed inside the low data within the '
-                    '\'kwarg\' key. See usage of '
-                    'salt.utils.args.parse_input() and '
-                    'salt.minion.load_args_and_kwargs() elsewhere in the '
-                    'codebase.'
-                )
-                kwargs = {}
-            else:
-                kwargs = low['kwarg']
+                kwargs = f_call.get('kwargs', {})
 
             # Update the event data with loaded args and kwargs
             data['fun_args'] = list(args) + ([kwargs] if kwargs else [])
@@ -405,8 +395,6 @@ class SyncClientMixin(object):
                     )
             data['success'] = False
 
-        namespaced_event.fire_event(data, 'ret')
-
         if self.store_job:
             try:
                 salt.utils.job.store_job(
@@ -424,6 +412,9 @@ class SyncClientMixin(object):
                 log.error('Could not store job cache info. '
                           'Job details for this run may be unavailable.')
 
+        # Outputters _can_ mutate data so write to the job cache first!
+        namespaced_event.fire_event(data, 'ret')
+
         # if we fired an event, make sure to delete the event object.
         # This will ensure that we call destroy, which will do the 0MQ linger
         log.info('Runner completed: {0}'.format(data['jid']))
