@@ -14,25 +14,25 @@ keys make the engine interactive.
     .. code-block:: yaml
 
         engines:
-          - hipchat:
-              api_url: http://api.hipchat.myteam.com
-              token: 'XXXXXX'
-              room: 'salt'
-              control: True
-              valid_users:
-                 - SomeUser
-              valid_commands:
-                 - test.ping
-                 - cmd.run
-                 - list_jobs
-                 - list_commands
-              aliases:
-                 list_jobs:
-                     cmd: jobs.list_jobs
-                 list_commands:
-                     cmd: pillar.get salt:engines:hipchat:valid_commands target=saltmaster tgt_type=list
-              max_rooms: 0
-              wait_time: 1
+            - hipchat:
+                api_url: http://api.hipchat.myteam.com
+                token: 'XXXXXX'
+                room: 'salt'
+                control: True
+                valid_users:
+                    - SomeUser
+                valid_commands:
+                    - test.ping
+                    - cmd.run
+                    - list_jobs
+                    - list_commands
+                aliases:
+                    list_jobs:
+                        cmd: jobs.list_jobs
+                    list_commands:
+                        cmd: pillar.get salt:engines:hipchat:valid_commands target=saltmaster
+                max_rooms: 0
+                wait_time: 1
 '''
 
 from __future__ import absolute_import
