@@ -621,6 +621,13 @@ class Client(object):
 
             def on_header(hdr):
                 if write_body[1] is not False and write_body[2] is None:
+                    if not hdr.strip() and 'Content-Type' not in write_body[1]:
+                        # We've reached the end of the headers and not yet
+                        # found the Content-Type. Reset the values we're
+                        # tracking so that we properly follow the redirect.
+                        write_body[0] = None
+                        write_body[1] = False
+                        return
                     # Try to find out what content type encoding is used if
                     # this is a text file
                     write_body[1].parse_line(hdr)  # pylint: disable=no-member
@@ -1257,16 +1264,16 @@ class RemoteClient(Client):
             if not os.path.isfile(path):
                 msg = 'specified file {0} is not present to generate hash: {1}'
                 log.warning(msg.format(path, err))
-                return {}
+                return {}, None
             else:
                 ret = {}
                 hash_type = self.opts.get('hash_type', 'md5')
                 ret['hsum'] = salt.utils.get_hash(path, form=hash_type)
                 ret['hash_type'] = hash_type
-                return ret, list(os.stat(path))
+                return ret
         load = {'path': path,
                 'saltenv': saltenv,
-                'cmd': '_file_hash_and_stat'}
+                'cmd': '_file_hash'}
         return self.channel.send(load)
 
     def hash_file(self, path, saltenv='base'):
@@ -1275,14 +1282,33 @@ class RemoteClient(Client):
         master file server prepend the path with salt://<file on server>
         otherwise, prepend the file with / for a local file.
         '''
-        return self.__hash_and_stat_file(path, saltenv)[0]
+        return self.__hash_and_stat_file(path, saltenv)
 
     def hash_and_stat_file(self, path, saltenv='base'):
         '''
         The same as hash_file, but also return the file's mode, or None if no
         mode data is present.
         '''
-        return self.__hash_and_stat_file(path, saltenv)
+        hash_result = self.hash_file(path, saltenv)
+        try:
+            path = self._check_proto(path)
+        except MinionError as err:
+            if not os.path.isfile(path):
+                return hash_result, None
+            else:
+                try:
+                    return hash_result, list(os.stat(path))
+                except Exception:
+                    return hash_result, None
+        load = {'path': path,
+                'saltenv': saltenv,
+                'cmd': '_file_find'}
+        fnd = self.channel.send(load)
+        try:
+            stat_result = fnd.get('stat')
+        except AttributeError:
+            stat_result = None
+        return hash_result, stat_result
 
     def list_env(self, saltenv='base'):
         '''
