@@ -48,20 +48,20 @@ Connection module for Amazon RDS
 # Import Python libs
 from __future__ import absolute_import
 import logging
-from salt.exceptions import SaltInvocationError
-from distutils.version import LooseVersion as _LooseVersion  # pylint: disable=import-error,no-name-in-module
-from time import time, sleep
+import time
 
 # Import Salt libs
 import salt.utils.boto3
 import salt.utils.compat
 import salt.utils.odict as odict
 import salt.utils
-import salt.ext.six as six
+from salt.exceptions import SaltInvocationError
+from salt.utils.versions import LooseVersion as _LooseVersion
 
 log = logging.getLogger(__name__)
 
 # Import third party libs
+import salt.ext.six as six
 # pylint: disable=import-error
 try:
     #pylint: disable=unused-import
@@ -298,7 +298,7 @@ def create(name, allocated_storage, db_instance_class, engine,
 
         while True:
             log.info('Waiting 10 secs...')
-            sleep(10)
+            time.sleep(10)
             _describe = describe(name=name, tags=tags, region=region, key=key,
                                  keyid=keyid, profile=profile)['rds']
             if not _describe:
@@ -570,31 +570,27 @@ def get_endpoint(name, tags=None, region=None, key=None, keyid=None,
         salt myminion boto_rds.get_endpoint myrds
 
     '''
-    endpoint = 'None'
+    endpoint = False
     res = __salt__['boto_rds.exists'](name, tags, region, key, keyid,
                                       profile)
-    if not res:
-        return {'exists': bool(res), 'message':
-                'RDS instance {0} does not exist.'.format(name)}
+    if res.get('exists'):
+        try:
+            conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+            if conn:
+                rds = conn.describe_db_instances(DBInstanceIdentifier=name)
 
-    try:
-        conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
-        if not conn:
-            return {'results': bool(conn)}
+                if rds and 'Endpoint' in rds['DBInstances'][0]:
+                    endpoint = rds['DBInstances'][0]['Endpoint']['Address']
+                    return endpoint
 
-        rds = conn.describe_db_instances(DBInstanceIdentifier=name)
+        except ClientError as e:
+            return {'error': salt.utils.boto3.get_error(e)}
 
-        if rds:
-            inst = rds['DBInstances'][0]['Endpoint']
-            endpoint = '{0}:{1}'.format(inst.get('Address'), inst.get('Port'))
-            return endpoint
-
-    except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+    return endpoint
 
 
 def delete(name, skip_final_snapshot=None, final_db_snapshot_identifier=None,
-           region=None, key=None, keyid=None, profile=None,
+           region=None, key=None, keyid=None, profile=None, tags=None,
            wait_for_deletion=True, timeout=180):
     '''
     Delete an RDS instance.
@@ -630,18 +626,20 @@ def delete(name, skip_final_snapshot=None, final_db_snapshot_identifier=None,
             return {'deleted': bool(res), 'message':
                     'Deleted RDS instance {0}.'.format(name)}
 
-        start_time = time()
+        start_time = time.time()
         while True:
-            res = __salt__['boto_rds.exists'](name=name, region=region, key=key, keyid=keyid, profile=profile)
+            res = __salt__['boto_rds.exists'](name=name, tags=tags, region=region,
+                                               key=key, keyid=keyid,
+                                               profile=profile)
             if not res.get('exists'):
                 return {'deleted': bool(res), 'message':
                         'Deleted RDS instance {0} completely.'.format(name)}
 
-            if time() - start_time > timeout:
+            if time.time() - start_time > timeout:
                 raise SaltInvocationError('RDS instance {0} has not been '
                                           'deleted completely after {1} '
                                           'seconds'.format(name, timeout))
-            sleep(10)
+            time.sleep(10)
     except ClientError as e:
         return {'error': salt.utils.boto3.get_error(e)}
 
