@@ -124,7 +124,6 @@ except ImportError:
     USE_FILTERCLASS = False
 
 MIN_DOCKERCOMPOSE = (1, 5, 0)
-MAX_DOCKERCOMPOSE = (1, 9, 0)
 VERSION_RE = r'([\d.]+)'
 
 log = logging.getLogger(__name__)
@@ -139,7 +138,7 @@ def __virtual__():
         match = re.match(VERSION_RE, str(compose.__version__))
         if match:
             version = tuple([int(x) for x in match.group(1).split('.')])
-            if version >= MIN_DOCKERCOMPOSE and version <= MAX_DOCKERCOMPOSE:
+            if version >= MIN_DOCKERCOMPOSE:
                 return __virtualname__
     return (False, 'The dockercompose execution module not loaded: '
             'compose python library not available.')
@@ -180,7 +179,7 @@ def __read_docker_compose(path):
         return __standardize_result(False,
                                     'Path does not exist or docker-compose.yml is not present',
                                     None, None)
-    f = salt.utils.fopen(os.path.join(path, dc_filename), 'r')
+    f = salt.utils.fopen(os.path.join(path, dc_filename), 'r')  # pylint: disable=resource-leakage
     result = {'docker-compose.yml': ''}
     if f:
         for line in f:
@@ -208,7 +207,7 @@ def __write_docker_compose(path, docker_compose):
 
     if os.path.isdir(path) is False:
         os.mkdir(path)
-    f = salt.utils.fopen(os.path.join(path, dc_filename), 'w')
+    f = salt.utils.fopen(os.path.join(path, dc_filename), 'w')  # pylint: disable=resource-leakage
     if f:
         f.write(docker_compose)
         f.close()
