@@ -80,7 +80,7 @@ def _mine_get(load, opts):
     return ret
 
 
-def update(clear=False):
+def update(clear=False, mine_functions=None):
     '''
     Execute the configured functions and send the data back up to the master.
     The functions to be executed are merged from the master config, pillar and
@@ -93,6 +93,34 @@ def update(clear=False):
             - eth0
           disk.usage: []
 
+    This function accepts the following arguments:
+
+    clear: False
+        Boolean flag specifying whether updating will clear the existing
+        mines, or will update. Default: `False` (update).
+
+    mine_functions
+        Update the mine data on certain functions only.
+        This feature can be used when updating the mine for functions
+        that require refresh at different intervals than the rest of
+        the functions specified under `mine_functions` in the
+        minion/master config or pillar.
+        A potential use would be together with the `scheduler`, for example:
+
+        .. code-block:: yaml
+
+            schedule:
+              lldp_mine_update:
+                function: mine.update
+                kwargs:
+                    mine_functions:
+                      net.lldp: []
+                hours: 12
+
+        In the example above, the mine for `net.lldp` would be refreshed
+        every 12 hours, while  `network.ip_addrs` would continue to be updated
+        as specified in `mine_interval`.
+
     The function cache will be populated with information from executing these
     functions
 
@@ -102,9 +130,17 @@ def update(clear=False):
 
         salt '*' mine.update
     '''
-    m_data = __salt__['config.merge']('mine_functions', {})
-    # If we don't have any mine functions configured, then we should just bail out
-    if not m_data:
+    m_data = {}
+    if not mine_functions:
+        m_data = __salt__['config.merge']('mine_functions', {})
+        # If we don't have any mine functions configured, then we should just bail out
+        if not m_data:
+            return
+    elif mine_functions and isinstance(mine_functions, list):
+        m_data = dict((fun, {}) for fun in mine_functions)
+    elif mine_functions and isinstance(mine_functions, dict):
+        m_data = mine_functions
+    else:
         return
 
     data = {}
@@ -203,20 +239,24 @@ def send(func, *args, **kwargs):
     return _mine_send(load, __opts__)
 
 
-def get(tgt, fun, expr_form='glob', exclude_minion=False):
+def get(tgt,
+        fun,
+        tgt_type='glob',
+        exclude_minion=False,
+        expr_form=None):
     '''
-    Get data from the mine based on the target, function and expr_form
+    Get data from the mine based on the target, function and tgt_type
 
     Targets can be matched based on any standard matching system that can be
-    matched on the master via these keywords::
+    matched on the master via these keywords:
 
-        glob
-        pcre
-        grain
-        grain_pcre
-        compound
-        pillar
-        pillar_pcre
+    - glob
+    - pcre
+    - grain
+    - grain_pcre
+    - compound
+    - pillar
+    - pillar_pcre
 
     Note that all pillar matches, whether using the compound matching system or
     the pillar matching system, will be exact matches, with globbing disabled.
@@ -237,7 +277,7 @@ def get(tgt, fun, expr_form='glob', exclude_minion=False):
         This execution module is intended to be executed on minions.
         Master-side operations such as Pillar or Orchestrate that require Mine
         data should use the :py:mod:`Mine Runner module <salt.runners.mine>`
-        instead; it can be invoked from an SLS file using the
+        instead; it can be invoked from a Pillar SLS file using the
         :py:func:`saltutil.runner <salt.modules.saltutil.runner>` module. For
         example:
 
@@ -248,6 +288,17 @@ def get(tgt, fun, expr_form='glob', exclude_minion=False):
                 fun='network.ip_addrs',
                 tgt_type='glob') %}
     '''
+    # remember to remove the expr_form argument from this function when
+    # performing the cleanup on this deprecation.
+    if expr_form is not None:
+        salt.utils.warn_until(
+            'Fluorine',
+            'the target type should be passed using the \'tgt_type\' '
+            'argument instead of \'expr_form\'. Support for using '
+            '\'expr_form\' will be removed in Salt Fluorine.'
+        )
+        tgt_type = expr_form
+
     if __opts__['file_client'] == 'local':
         ret = {}
         is_target = {'glob': __salt__['match.glob'],
@@ -259,7 +310,7 @@ def get(tgt, fun, expr_form='glob', exclude_minion=False):
                      'compound': __salt__['match.compound'],
                      'pillar': __salt__['match.pillar'],
                      'pillar_pcre': __salt__['match.pillar_pcre'],
-                     }[expr_form](tgt)
+                     }[tgt_type](tgt)
         if is_target:
             data = __salt__['data.get']('mine_cache')
             if isinstance(data, dict) and fun in data:
@@ -270,7 +321,7 @@ def get(tgt, fun, expr_form='glob', exclude_minion=False):
             'id': __opts__['id'],
             'tgt': tgt,
             'fun': fun,
-            'expr_form': expr_form,
+            'tgt_type': tgt_type,
     }
     ret = _mine_get(load, __opts__)
     if exclude_minion:
@@ -356,7 +407,7 @@ def get_docker(interfaces=None, cidrs=None, with_container_id=False):
         cidrs = cidr_
 
     # Get docker info
-    cmd = 'dockerng.ps'
+    cmd = 'docker.ps'
     docker_hosts = get('*', cmd)
 
     proxy_lists = {}
