@@ -150,7 +150,9 @@ class Registry(object):  # pylint: disable=R0903
             _winreg.REG_DWORD:     'REG_DWORD',
             _winreg.REG_EXPAND_SZ: 'REG_EXPAND_SZ',
             _winreg.REG_MULTI_SZ:  'REG_MULTI_SZ',
-            _winreg.REG_SZ:        'REG_SZ'
+            _winreg.REG_SZ:        'REG_SZ',
+            # REG_QWORD isn't in the winreg library
+            11:                    'REG_QWORD'
         }
         self.opttype_reverse = {
             _winreg.REG_OPTION_NON_VOLATILE: 'REG_OPTION_NON_VOLATILE',
