@@ -35,6 +35,16 @@ Connection module for Amazon S3
     The service_url will form the basis for the final endpoint that is used to
     query the service.
 
+    Path style can be enabled:
+
+        s3.path_style: True
+
+    This can be useful if you need to use salt with a proxy for an s3 compatible storage
+
+    You can use either https protocol or http protocol:
+
+        s3.https_enable: True
+
     SSL verification may also be turned off in the configuration:
 
         s3.verify_ssl: False
@@ -71,7 +81,7 @@ def __virtual__():
 
 def delete(bucket, path=None, action=None, key=None, keyid=None,
            service_url=None, verify_ssl=None, kms_keyid=None, location=None,
-           role_arn=None):
+           role_arn=None, path_style=None, https_enable=None):
     '''
     Delete a bucket, or delete an object from a bucket.
 
@@ -83,7 +93,7 @@ def delete(bucket, path=None, action=None, key=None, keyid=None,
 
         salt myminion s3.delete mybucket remoteobject
     '''
-    key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn = _get_key(
+    key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn, path_style, https_enable = _get_key(
         key,
         keyid,
         service_url,
@@ -91,6 +101,8 @@ def delete(bucket, path=None, action=None, key=None, keyid=None,
         kms_keyid,
         location,
         role_arn,
+        path_style,
+        https_enable,
     )
 
     return __utils__['s3.query'](method='DELETE',
@@ -103,12 +115,15 @@ def delete(bucket, path=None, action=None, key=None, keyid=None,
                                  service_url=service_url,
                                  verify_ssl=verify_ssl,
                                  location=location,
-                                 role_arn=role_arn)
+                                 role_arn=role_arn,
+                                 path_style=path_style,
+                                 https_enable=https_enable)
 
 
 def get(bucket='', path='', return_bin=False, action=None,
         local_file=None, key=None, keyid=None, service_url=None,
-        verify_ssl=None, kms_keyid=None, location=None, role_arn=None):
+        verify_ssl=None, kms_keyid=None, location=None, role_arn=None,
+        path_style=None, https_enable=None):
     '''
     List the contents of a bucket, or return an object from a bucket. Set
     return_bin to True in order to retrieve an object wholesale. Otherwise,
@@ -160,7 +175,7 @@ def get(bucket='', path='', return_bin=False, action=None,
 
         salt myminion s3.get mybucket myfile.png action=acl
     '''
-    key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn = _get_key(
+    key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn, path_style, https_enable = _get_key(
         key,
         keyid,
         service_url,
@@ -168,6 +183,8 @@ def get(bucket='', path='', return_bin=False, action=None,
         kms_keyid,
         location,
         role_arn,
+        path_style,
+        https_enable,
     )
 
     return __utils__['s3.query'](method='GET',
@@ -182,11 +199,14 @@ def get(bucket='', path='', return_bin=False, action=None,
                                  service_url=service_url,
                                  verify_ssl=verify_ssl,
                                  location=location,
-                                 role_arn=role_arn)
+                                 role_arn=role_arn,
+                                 path_style=path_style,
+                                 https_enable=https_enable)
 
 
 def head(bucket, path='', key=None, keyid=None, service_url=None,
-         verify_ssl=None, kms_keyid=None, location=None, role_arn=None):
+         verify_ssl=None, kms_keyid=None, location=None, role_arn=None,
+         path_style=None, https_enable=None):
     '''
     Return the metadata for a bucket, or an object in a bucket.
 
@@ -197,7 +217,7 @@ def head(bucket, path='', key=None, keyid=None, service_url=None,
         salt myminion s3.head mybucket
         salt myminion s3.head mybucket myfile.png
     '''
-    key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn = _get_key(
+    key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn, path_style, https_enable = _get_key(
         key,
         keyid,
         service_url,
@@ -205,6 +225,8 @@ def head(bucket, path='', key=None, keyid=None, service_url=None,
         kms_keyid,
         location,
         role_arn,
+        path_style,
+        https_enable,
     )
 
     return __utils__['s3.query'](method='HEAD',
@@ -217,12 +239,15 @@ def head(bucket, path='', key=None, keyid=None, service_url=None,
                                  verify_ssl=verify_ssl,
                                  location=location,
                                  full_headers=True,
-                                 role_arn=role_arn)
+                                 role_arn=role_arn,
+                                 path_style=path_style,
+                                 https_enable=https_enable)
 
 
 def put(bucket, path=None, return_bin=False, action=None, local_file=None,
         key=None, keyid=None, service_url=None, verify_ssl=None,
-        kms_keyid=None, location=None, role_arn=None):
+        kms_keyid=None, location=None, role_arn=None, path_style=None,
+        https_enable=None):
     '''
     Create a new bucket, or upload an object to a bucket.
 
@@ -238,7 +263,7 @@ def put(bucket, path=None, return_bin=False, action=None, local_file=None,
 
         salt myminion s3.put mybucket remotepath local_file=/path/to/file
     '''
-    key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn = _get_key(
+    key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn, path_style, https_enable = _get_key(
         key,
         keyid,
         service_url,
@@ -246,6 +271,8 @@ def put(bucket, path=None, return_bin=False, action=None, local_file=None,
         kms_keyid,
         location,
         role_arn,
+        path_style,
+        https_enable,
     )
 
     return __utils__['s3.query'](method='PUT',
@@ -260,10 +287,12 @@ def put(bucket, path=None, return_bin=False, action=None, local_file=None,
                                  service_url=service_url,
                                  verify_ssl=verify_ssl,
                                  location=location,
-                                 role_arn=role_arn)
+                                 role_arn=role_arn,
+                                 path_style=path_style,
+                                 https_enable=https_enable)
 
 
-def _get_key(key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn):
+def _get_key(key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn, path_style, https_enable):
     '''
     Examine the keys, and populate as necessary
     '''
@@ -294,4 +323,16 @@ def _get_key(key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn)
     if role_arn is None and __salt__['config.option']('s3.role_arn'):
         role_arn = __salt__['config.option']('s3.role_arn')
 
-    return key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn
+    if path_style is None and __salt__['config.option']('s3.path_style') is not None:
+        path_style = __salt__['config.option']('s3.path_style')
+
+    if path_style is None:
+        path_style = False
+
+    if https_enable is None and __salt__['config.option']('s3.https_enable') is not None:
+        https_enable = __salt__['config.option']('s3.https_enable')
+
+    if https_enable is None:
+        https_enable = True
+
+    return key, keyid, service_url, verify_ssl, kms_keyid, location, role_arn, path_style, https_enable
