@@ -3,7 +3,12 @@
 Module for managing windows systems.
 
 :depends:
+    - pythoncom
+    - pywintypes
+    - win32api
+    - win32con
     - win32net
+    - wmi
 
 Support for reboot, shutdown, etc
 '''
@@ -12,6 +17,7 @@ from __future__ import absolute_import
 # Import python libs
 import logging
 import time
+import ctypes
 from datetime import datetime
 
 # Import 3rd Party Libs
@@ -22,7 +28,6 @@ try:
     import win32api
     import win32con
     import pywintypes
-    import ctypes
     from ctypes import windll
     HAS_WIN32NET_MODS = True
 except ImportError:
@@ -32,6 +37,7 @@ except ImportError:
 import salt.utils
 import salt.utils.locales
 import salt.ext.six as six
+from salt.exceptions import CommandExecutionError
 
 # Set up logging
 log = logging.getLogger(__name__)
@@ -42,11 +48,15 @@ __virtualname__ = 'system'
 
 def __virtual__():
     '''
-    Set the system module of the kernel is Windows
+    Only works on Windows Systems with Win32 Modules
     '''
-    if HAS_WIN32NET_MODS and salt.utils.is_windows():
-        return __virtualname__
-    return (False, "Module win_system: module only works on Windows systems")
+    if not salt.utils.is_windows():
+        return False, 'Module win_system: Requires Windows'
+
+    if not HAS_WIN32NET_MODS:
+        return False, 'Module win_system: Missing win32 modules'
+
+    return __virtualname__
 
 
 def _convert_minutes_seconds(timeout, in_seconds=False):
@@ -65,24 +75,32 @@ def _convert_date_time_string(dt_string):
     return dt_obj.strftime('%Y-%m-%d %H:%M:%S')
 
 
+def _to_unicode(instr):
+    '''
+    Converts from current users character encoding to unicode.
+    When instr has a value of None, the return value of the function
+    will also be None.
+    '''
+    if instr is None or isinstance(instr, six.text_type):
+        return instr
+    else:
+        return six.text_type(instr, 'utf8')
+
+
 def halt(timeout=5, in_seconds=False):
     '''
     Halt a running system.
 
-    :param int timeout:
-        Number of seconds before halting the system.
-        Default is 5 seconds.
+    Args:
+        timeout (int): Number of seconds before halting the system. Default is
+        5 seconds.
 
-    :return: True is successful.
-    :rtype: bool
+        in_seconds (bool): Whether to treat timeout as seconds or minutes.
 
-    timeout
-        The wait time before the system will be shutdown.
+            .. versionadded:: 2015.8.0
 
-    in_seconds
-        Whether to treat timeout as seconds or minutes.
-
-        .. versionadded:: 2015.8.0
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -117,20 +135,15 @@ def poweroff(timeout=5, in_seconds=False):
     '''
     Power off a running system.
 
-    :param int timeout:
-        Number of seconds before powering off the system.
-        Default is 5 seconds.
+    Args:
+        timeout (int): Number of seconds before powering off the system. Default
+            is 5 seconds.
 
-    :return: True if successful
-    :rtype: bool
+        in_seconds (bool): Whether to treat timeout as seconds or minutes.
+            .. versionadded:: 2015.8.0
 
-    timeout
-        The wait time before the system will be shutdown.
-
-    in_seconds
-        Whether to treat timeout as seconds or minutes.
-
-        .. versionadded:: 2015.8.0
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -146,35 +159,30 @@ def reboot(timeout=5, in_seconds=False, wait_for_reboot=False,  # pylint: disabl
     '''
     Reboot a running system.
 
-    :param int timeout:
-        Number of minutes/seconds before rebooting the system. Minutes vs
-        seconds depends on the value of ``in_seconds``.
-        Default is 5 minutes.
+    Args:
+        timeout (int): Number of minutes/seconds before rebooting the system.
+            Minutes vs seconds depends on the value of ``in_seconds``. Default
+            is 5 minutes.
 
-    :param bool in_seconds:
-        Whether to treat timeout as seconds or minutes.
+        in_seconds (bool): Whether to treat timeout as seconds or minutes.
 
-        .. versionadded:: 2015.8.0
+            .. versionadded:: 2015.8.0
 
-    :param bool wait_for_reboot:
+        wait_for_reboot (bool): Sleeps for timeout + 30 seconds after reboot has
+            been initiated. This may be useful for use in a highstate if a
+            reboot should be performed and the return data of the highstate is
+            not required. If return data is required, consider using the reboot
+            state instead of this module.
 
-        Sleeps for timeout + 30 seconds after reboot has been initiated.
-        This is useful for use in a highstate for example where
-        you have many states that could be ran after this one. Which you don't want
-        to start until after the restart i.e You could end up with a half finished state.
+            .. versionadded:: 2015.8.0
 
-        .. versionadded:: 2015.8.0
+        only_on_pending_reboot (bool): If this is set to True, then the reboot
+            will only proceed if the system reports a pending reboot. To
+            optionally reboot in a highstate, consider using the reboot state
+            instead of this module.
 
-    :param bool only_on_pending_reboot:
-
-        If this is set to True, then then the reboot will only proceed
-        if the system reports a pending reboot. Setting this parameter to
-        True could be useful when calling this function from a final housekeeping
-        state intended to be executed
-        at the end of a state run (using *order: last*).
-
-    :return: True if successful (a reboot will occur)
-    :rtype: bool
+    Returns:
+        bool: True if successful (a reboot will occur), otherwise False
 
     CLI Example:
 
@@ -195,7 +203,6 @@ def reboot(timeout=5, in_seconds=False, wait_for_reboot=False,  # pylint: disabl
               - name: system.reboot
               - only_on_pending_reboot: True
               - order: last
-
     '''
 
     ret = shutdown(timeout=timeout, reboot=True, in_seconds=in_seconds,
@@ -213,46 +220,45 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False,  # pylint:
     '''
     Shutdown a running system.
 
-    :param str message:
-        A message to display to the user before shutting down.
+    Args:
+        message (str): A message to display to the user before shutting down.
 
-    :param int timeout:
-        The length of time that the shutdown dialog box should be displayed, in
-        seconds. While this dialog box is displayed, the shutdown can be stopped
-        by the shutdown_abort function.
+        timeout (int): The length of time that the shutdown dialog box should be
+            displayed, in seconds. While this dialog box is displayed, the
+            shutdown can be stopped by the shutdown_abort function.
 
-        If timeout is not zero, InitiateSystemShutdown displays a dialog box on
-        the specified computer. The dialog box displays the name of the user
-        who called the function, displays the message specified by the
-        lpMessage parameter, and prompts the user to log off. The dialog box
-        beeps when it is created and remains on top of other windows in the
-        system. The dialog box can be moved but not closed. A timer counts down
-        the remaining time before a forced shutdown.
+            If timeout is not zero, InitiateSystemShutdown displays a dialog box
+            on the specified computer. The dialog box displays the name of the
+            user who called the function, displays the message specified by the
+            lpMessage parameter, and prompts the user to log off. The dialog box
+            beeps when it is created and remains on top of other windows in the
+            system. The dialog box can be moved but not closed. A timer counts
+            down the remaining time before a forced shutdown.
 
-        If timeout is zero, the computer shuts down without displaying the
-        dialog box, and the shutdown cannot be stopped by shutdown_abort.
+            If timeout is zero, the computer shuts down without displaying the
+            dialog box, and the shutdown cannot be stopped by shutdown_abort.
 
-        Default is 5 minutes
+            Default is 5 minutes
 
-    :param bool in_seconds:
-        Whether to treat timeout as seconds or minutes.
+        in_seconds (bool): Whether to treat timeout as seconds or minutes.
 
-        .. versionadded:: 2015.8.0
+            .. versionadded:: 2015.8.0
 
-    :param bool force_close:
-        True to force close all open applications. False displays a dialog box
-        instructing the user to close the applications.
+        force_close (bool): True to force close all open applications. False
+            displays a dialog box instructing the user to close the
+            applications.
 
-    :param bool reboot:
-        True restarts the computer immediately after shutdown.
-        False caches to disk and safely powers down the system.
+        reboot (bool): True restarts the computer immediately after shutdown.
+            False caches to disk and safely powers down the system.
 
-    :param bool only_on_pending_reboot:
-        If this is set to True, then then shutdown will only proceed
-        if the system reports a pending reboot.
+        only_on_pending_reboot (bool): If this is set to True, then the shutdown
+            will only proceed if the system reports a pending reboot. To
+            optionally shutdown in a highstate, consider using the shutdown
+            state instead of this module.
 
-    :return: True if successful (a shutdown or reboot will occur)
-    :rtype: bool
+    Returns:
+        bool: True if successful (a shutdown or reboot will occur), otherwise
+            False
 
     CLI Example:
 
@@ -260,6 +266,9 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False,  # pylint:
 
         salt '*' system.shutdown 5
     '''
+    if six.PY2:
+        message = _to_unicode(message)
+
     timeout = _convert_minutes_seconds(timeout, in_seconds)
 
     if only_on_pending_reboot and not get_pending_reboot():
@@ -284,8 +293,8 @@ def shutdown_hard():
     '''
     Shutdown a running system with no timeout or warning.
 
-    :return: True if successful
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -299,10 +308,11 @@ def shutdown_hard():
 def shutdown_abort():
     '''
     Abort a shutdown. Only available while the dialog box is being
-    displayed to the user. Once the shutdown has initiated, it cannot be aborted
+    displayed to the user. Once the shutdown has initiated, it cannot be
+    aborted.
 
-    :return: True if successful
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -326,8 +336,8 @@ def lock():
     '''
     Lock the workstation.
 
-    :return: True if successful
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -342,12 +352,13 @@ def set_computer_name(name):
     '''
     Set the Windows computer name
 
-    :param str name:
-        The new name to give the computer. Requires a reboot to take effect.
+    Args:
+        name (str): The new name to give the computer. Requires a reboot to take
+            effect.
 
-    :return:
-        Returns a dictionary containing the old and new names if successful.
-        False if not.
+    Returns:
+        dict: Returns a dictionary containing the old and new names if
+            successful. False if not.
 
     CLI Example:
 
@@ -355,16 +366,17 @@ def set_computer_name(name):
 
         salt 'minion-id' system.set_computer_name 'DavesComputer'
     '''
-    if name and six.PY2:
-        name = name.decode('utf-8')
+    if six.PY2:
+        name = _to_unicode(name)
 
-    if windll.kernel32.SetComputerNameExW(win32con.ComputerNamePhysicalDnsHostname,
-                                          name):
+    if windll.kernel32.SetComputerNameExW(
+            win32con.ComputerNamePhysicalDnsHostname, name):
         ret = {'Computer Name': {'Current': get_computer_name()}}
         pending = get_pending_computer_name()
         if pending not in (None, False):
             ret['Computer Name']['Pending'] = pending
         return ret
+
     return False
 
 
@@ -376,9 +388,8 @@ def get_pending_computer_name():
     retrieving the pending computer name, ``False`` will be returned, and an
     error message will be logged to the minion log.
 
-    :return:
-        Returns the pending name if pending restart. Returns none if not pending
-        restart.
+    Returns:
+        str: The pending name if restart is pending, otherwise returns None.
 
     CLI Example:
 
@@ -400,8 +411,8 @@ def get_computer_name():
     '''
     Get the Windows computer name
 
-    :return:
-        Returns the computer name if found. Otherwise returns False
+    Returns:
+        str: Returns the computer name if found. Otherwise returns False
 
     CLI Example:
 
@@ -417,10 +428,11 @@ def set_computer_desc(desc=None):
     '''
     Set the Windows computer description
 
-    :param str desc:
-        The computer description
+    Args:
+        desc (str): The computer description
 
-    :return: False if it fails. Description if successful.
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -428,6 +440,9 @@ def set_computer_desc(desc=None):
 
         salt 'minion-id' system.set_computer_desc 'This computer belongs to Dave!'
     '''
+    if six.PY2:
+        desc = _to_unicode(desc)
+
     # Make sure the system exists
     # Return an object containing current information array for the computer
     system_info = win32net.NetServerGetInfo(None, 101)
@@ -435,8 +450,7 @@ def set_computer_desc(desc=None):
     # If desc is passed, decode it for unicode
     if desc is None:
         return False
-    if not isinstance(desc, str):
-        desc = desc.decode('utf-8')
+
     system_info['comment'] = desc
 
     # Apply new settings
@@ -460,10 +474,9 @@ def get_system_info():
     '''
     Get system information.
 
-    :return:
-        Returns a Dictionary containing information about the system to include
-        name, description, version, etc...
-    :rtype: dict
+    Returns:
+        dict: Returns a Dictionary containing information about the system to
+            include name, description, version, etc...
 
     CLI Example:
 
@@ -514,8 +527,9 @@ def get_system_info():
 def get_computer_desc():
     '''
     Get the Windows computer description
-    :return:
-        Returns the computer description if found. Otherwise returns False
+
+    Returns:
+        str: The computer description if found, otherwise False
 
     CLI Example:
 
@@ -536,8 +550,8 @@ def get_hostname():
 
     Get the hostname of the windows minion
 
-    :return:
-        Returns the hostname of the windows minion
+    Returns:
+        str: The hostname of the windows minion
 
     CLI Example:
 
@@ -557,8 +571,11 @@ def set_hostname(hostname):
     Set the hostname of the windows minion, requires a restart before this
     will be updated.
 
-    :param str hostname:
-        The hostname to set
+    Args:
+        hostname (str): The hostname to set
+
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -582,35 +599,35 @@ def join_domain(domain,
     '''
     Join a computer to an Active Directory domain. Requires reboot.
 
-    :param str domain:
-        The domain to which the computer should be joined, e.g.
-        ``example.com``
+    Args:
+        domain (str): The domain to which the computer should be joined, e.g.
+            ``example.com``
 
-    :param str username:
-        Username of an account which is authorized to join computers to the
-        specified domain. Need to be either fully qualified like
-        ``user@domain.tld`` or simply ``user``
+        username (str): Username of an account which is authorized to join
+            computers to the specified domain. Need to be either fully qualified
+            like ``user@domain.tld`` or simply ``user``
 
-    :param str password:
-        Password of the specified user
+        password (str): Password of the specified user
 
-    :param str account_ou:
-        The DN of the OU below which the account for this computer should be
-        created when joining the domain, e.g.
-        ``ou=computers,ou=departm_432,dc=my-company,dc=com``
+        account_ou (str): The DN of the OU below which the account for this
+            computer should be created when joining the domain, e.g.
+            ``ou=computers,ou=departm_432,dc=my-company,dc=com``
 
-    :param bool account_exists:
-        If set to ``True`` the computer will only join the domain if the account
-        already exists. If set to ``False`` the computer account will be created
-        if it does not exist, otherwise it will use the existing account.
-        Default is False
+        account_exists (bool): If set to ``True`` the computer will only join
+            the domain if the account already exists. If set to ``False`` the
+            computer account will be created if it does not exist, otherwise it
+            will use the existing account. Default is False.
 
-    :param bool restart: Restarts the computer after a successful join
+        restart (bool): Restarts the computer after a successful join
 
-        .. versionadded:: 2015.8.2/2015.5.7
+            .. versionadded:: 2015.8.2/2015.5.7
 
-    :returns: Returns a dictionary if successful. False if unsuccessful.
-    :rtype: dict, bool
+    Returns:
+        dict: Dictionary if successful
+
+    Raises:
+        CommandExecutionError: Raises an error if _join_domain returns anything
+            other than 0
 
     CLI Example:
 
@@ -621,6 +638,12 @@ def join_domain(domain,
                          account_ou='ou=clients,ou=org,dc=domain,dc=tld' \\
                          account_exists=False, restart=True
     '''
+    if six.PY2:
+        domain = _to_unicode(domain)
+        username = _to_unicode(username)
+        password = _to_unicode(password)
+        account_ou = _to_unicode(account_ou)
+
     status = get_domain_workgroup()
     if 'Domain' in status:
         if status['Domain'] == domain:
@@ -637,6 +660,56 @@ def join_domain(domain,
         account_ou = account_ou.split('\\')
         account_ou = ''.join(account_ou)
 
+    err = _join_domain(domain=domain, username=username, password=password,
+                       account_ou=account_ou, account_exists=account_exists)
+
+    if not err:
+        ret = {'Domain': domain,
+               'Restart': False}
+        if restart:
+            ret['Restart'] = reboot()
+        return ret
+
+    raise CommandExecutionError(win32api.FormatMessage(err).rstrip())
+
+
+def _join_domain(domain,
+                 username=None,
+                 password=None,
+                 account_ou=None,
+                 account_exists=False):
+    '''
+    Helper function to join the domain.
+
+    Args:
+        domain (str): The domain to which the computer should be joined, e.g.
+            ``example.com``
+
+        username (str): Username of an account which is authorized to join
+            computers to the specified domain. Need to be either fully qualified
+            like ``user@domain.tld`` or simply ``user``
+
+        password (str): Password of the specified user
+
+        account_ou (str): The DN of the OU below which the account for this
+            computer should be created when joining the domain, e.g.
+            ``ou=computers,ou=departm_432,dc=my-company,dc=com``
+
+        account_exists (bool): If set to ``True`` the computer will only join
+            the domain if the account already exists. If set to ``False`` the
+            computer account will be created if it does not exist, otherwise it
+            will use the existing account. Default is False.
+
+    Returns:
+        int:
+
+    :param domain:
+    :param username:
+    :param password:
+    :param account_ou:
+    :param account_exists:
+    :return:
+    '''
     NETSETUP_JOIN_DOMAIN = 0x1  # pylint: disable=invalid-name
     NETSETUP_ACCOUNT_CREATE = 0x2  # pylint: disable=invalid-name
     NETSETUP_DOMAIN_JOIN_IF_JOINED = 0x20  # pylint: disable=invalid-name
@@ -652,23 +725,13 @@ def join_domain(domain,
     pythoncom.CoInitialize()
     conn = wmi.WMI()
     comp = conn.Win32_ComputerSystem()[0]
-    err = comp.JoinDomainOrWorkgroup(Name=domain,
-                                     Password=password,
-                                     UserName=username,
-                                     AccountOU=account_ou,
-                                     FJoinOptions=join_options)
 
-    # you have to do this because JoinDomainOrWorkgroup returns a strangely
-    # formatted value that looks like (0,)
-    if not err[0]:
-        ret = {'Domain': domain,
-               'Restart': False}
-        if restart:
-            ret['Restart'] = reboot()
-        return ret
-
-    log.error(win32api.FormatMessage(err[0]).rstrip())
-    return False
+    # Return the results of the command as an error
+    # JoinDomainOrWorkgroup returns a strangely formatted value that looks like
+    # (0,) so return the first item
+    return comp.JoinDomainOrWorkgroup(
+        Name=domain, Password=password, UserName=username, AccountOU=account_ou,
+        FJoinOptions=join_options)[0]
 
 
 def unjoin_domain(username=None,
@@ -680,32 +743,31 @@ def unjoin_domain(username=None,
     r'''
     Unjoin a computer from an Active Directory Domain. Requires restart.
 
-    :param username:
-        Username of an account which is authorized to manage computer accounts
-        on the domain. Need to be fully qualified like ``user@domain.tld`` or
-        ``domain.tld\user``. If domain not specified, the passed domain will be
-        used. If computer account doesn't need to be disabled, can be None.
+    Args:
+        username (str): Username of an account which is authorized to manage
+            computer accounts on the domain. Need to be fully qualified like
+            ``user@domain.tld`` or ``domain.tld\user``. If domain not specified,
+            the passed domain will be used. If computer account doesn't need to
+            be disabled, can be None.
 
-    :param str password:
-        Password of the specified user
+        password (str): Password of the specified user
 
-    :param str domain: The domain from which to unjoin the computer. Can be None
+        domain (str): The domain from which to unjoin the computer. Can be None.
 
-    :param str workgroup: The workgroup to join the computer to. Default is
-    ``WORKGROUP``
+        workgroup (str): The workgroup to join the computer to. Default is
+            ``WORKGROUP``
 
-        .. versionadded:: 2015.8.2/2015.5.7
+            .. versionadded:: 2015.8.2/2015.5.7
 
-    :param bool disable:
-        Disable the computer account in Active Directory. True to disable.
-        Default is False
+        disable (bool): Disable the computer account in Active Directory. True
+            to disable. Default is False
 
-    :param bool restart: Restart the computer after successful unjoin
+        restart (bool): Restart the computer after successful unjoin
 
-        .. versionadded:: 2015.8.2/2015.5.7
+            .. versionadded:: 2015.8.2/2015.5.7
 
-    :returns: Returns a dictionary if successful. False if unsuccessful.
-    :rtype: dict, bool
+    Returns:
+        dict: Dictionary if successful, otherwise False
 
     CLI Example:
 
@@ -717,6 +779,11 @@ def unjoin_domain(username=None,
                          password='unjoinpassword' disable=True \\
                          restart=True
     '''
+    if six.PY2:
+        username = _to_unicode(username)
+        password = _to_unicode(password)
+        domain = _to_unicode(domain)
+
     status = get_domain_workgroup()
     if 'Workgroup' in status:
         if status['Workgroup'] == workgroup:
@@ -731,7 +798,7 @@ def unjoin_domain(username=None,
     if username and password is None:
         return 'Must specify a password if you pass a username'
 
-    NETSETUP_ACCT_DELETE = 0x2  # pylint: disable=invalid-name
+    NETSETUP_ACCT_DELETE = 0x4  # pylint: disable=invalid-name
 
     unjoin_options = 0x0
     if disable:
@@ -772,15 +839,14 @@ def get_domain_workgroup():
     .. versionadded:: 2015.5.7
     .. versionadded:: 2015.8.2
 
-    :return: The name of the domain or workgroup
-    :rtype: str
+    Returns:
+        str: The name of the domain or workgroup
 
     CLI Example:
 
     .. code-block:: bash
 
         salt 'minion-id' system.get_domain_workgroup
-
     '''
     pythoncom.CoInitialize()
     conn = wmi.WMI()
@@ -795,11 +861,13 @@ def _try_parse_datetime(time_str, fmts):
     '''
     Attempts to parse the input time_str as a date.
 
-    :param str time_str: A string representing the time
-    :param list fmts: A list of date format strings
+    Args:
+        time_str (str): A string representing the time
 
-    :return: Returns a datetime object if parsed properly. Otherwise None
-    :rtype datetime
+        fmts (list): A list of date format strings
+
+    Returns:
+        datetime: A datetime object if parsed properly, otherwise None
     '''
     result = None
     for fmt in fmts:
@@ -815,8 +883,8 @@ def get_system_time():
     '''
     Get the system time.
 
-    :return: Returns the system time in HH:MM:SS AM/PM format.
-    :rtype: str
+    Returns:
+        str: Returns the system time in HH:MM:SS AM/PM format.
 
     CLI Example:
 
@@ -841,15 +909,16 @@ def set_system_time(newtime):
     '''
     Set the system time.
 
-    :param str newtime:
-        The time to set. Can be any of the following formats.
-        - HH:MM:SS AM/PM
-        - HH:MM AM/PM
-        - HH:MM:SS (24 hour)
-        - HH:MM (24 hour)
+    Args:
+        newtime (str): The time to set. Can be any of the following formats.
 
-    :return: Returns True if successful. Otherwise False.
-    :rtype: bool
+            - HH:MM:SS AM/PM
+            - HH:MM AM/PM
+            - HH:MM:SS (24 hour)
+            - HH:MM (24 hour)
+
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -881,15 +950,25 @@ def set_system_date_time(years=None,
     element will be used. For example, if you don't pass the year, the current
     system year will be used. (Used by set_system_date and set_system_time)
 
-    :param int years: Years digit, ie: 2015
-    :param int months: Months digit: 1 - 12
-    :param int days: Days digit: 1 - 31
-    :param int hours: Hours digit: 0 - 23
-    :param int minutes: Minutes digit: 0 - 59
-    :param int seconds: Seconds digit: 0 - 59
+    Args:
+        years (int): Years digit, ie: 2015
 
-    :return: True if successful. Otherwise False.
-    :rtype: bool
+        months (int): Months digit: 1 - 12
+
+        days (int): Days digit: 1 - 31
+
+        hours (int): Hours digit: 0 - 23
+
+        minutes (int): Minutes digit: 0 - 59
+
+        seconds (int): Seconds digit: 0 - 59
+
+    Returns:
+        bool: True if successful
+
+    Raises:
+        CommandExecutionError: Raises an error if ``SetLocalTime`` function
+            fails
 
     CLI Example:
 
@@ -942,20 +1021,23 @@ def set_system_date_time(years=None,
         system_time.wSecond = int(seconds)
         system_time_ptr = ctypes.pointer(system_time)
         succeeded = ctypes.windll.kernel32.SetLocalTime(system_time_ptr)
-        return succeeded is not 0
-    except OSError:
+        if succeeded is not 0:
+            return True
+        else:
+            log.error('Failed to set local time')
+            raise CommandExecutionError(
+                win32api.FormatMessage(succeeded).rstrip())
+    except OSError as err:
         log.error('Failed to set local time')
-        return False
-
-    return True
+        raise CommandExecutionError(err)
 
 
 def get_system_date():
     '''
     Get the Windows system date
 
-    :return: Returns the system date.
-    :rtype: str
+    Returns:
+        str: The system date
 
     CLI Example:
 
@@ -971,14 +1053,15 @@ def set_system_date(newdate):
     '''
     Set the Windows system date. Use <mm-dd-yy> format for the date.
 
-    :param str newdate:
-        The date to set. Can be any of the following formats
-        - YYYY-MM-DD
-        - MM-DD-YYYY
-        - MM-DD-YY
-        - MM/DD/YYYY
-        - MM/DD/YY
-        - YYYY/MM/DD
+    Args:
+        newdate (str): The date to set. Can be any of the following formats:
+
+            - YYYY-MM-DD
+            - MM-DD-YYYY
+            - MM-DD-YY
+            - MM/DD/YYYY
+            - MM/DD/YY
+            - YYYY/MM/DD
 
     CLI Example:
 
@@ -1003,8 +1086,8 @@ def start_time_service():
     '''
     Start the Windows time service
 
-    :return: True if successful. Otherwise False
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False.
 
     CLI Example:
 
@@ -1019,8 +1102,8 @@ def stop_time_service():
     '''
     Stop the Windows time service
 
-    :return: True if successful. Otherwise False
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -1033,13 +1116,14 @@ def stop_time_service():
 
 def get_pending_component_servicing():
     '''
-    Determine whether there are pending Component Based Servicing tasks that require a reboot.
-
-    :return: A boolean representing whether there are pending Component Based Servicing tasks.
-    :rtype: bool
+    Determine whether there are pending Component Based Servicing tasks that
+    require a reboot.
 
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1064,11 +1148,11 @@ def get_pending_domain_join():
     '''
     Determine whether there is a pending domain join action that requires a reboot.
 
-    :return: A boolean representing whether there is a pending domain join action.
-    :rtype: bool
-
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1103,13 +1187,14 @@ def get_pending_domain_join():
 
 def get_pending_file_rename():
     '''
-    Determine whether there are pending file rename operations that require a reboot.
-
-    :return: A boolean representing whether there are pending file rename operations.
-    :rtype: bool
+    Determine whether there are pending file rename operations that require a
+    reboot.
 
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1137,13 +1222,14 @@ def get_pending_file_rename():
 
 def get_pending_servermanager():
     '''
-    Determine whether there are pending Server Manager tasks that require a reboot.
-
-    :return: A boolean representing whether there are pending Server Manager tasks.
-    :rtype: bool
+    Determine whether there are pending Server Manager tasks that require a
+    reboot.
 
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1176,11 +1262,11 @@ def get_pending_update():
     '''
     Determine whether there are pending updates that require a reboot.
 
-    :return: A boolean representing whether there are pending updates.
-    :rtype: bool
-
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1209,35 +1295,30 @@ REBOOT_REQUIRED_NAME = 'Reboot required'
 
 def set_reboot_required_witnessed():
     r'''
-    .. versionadded:: 2016.11.0
-
-    This function is used to remember that
-    an event indicating that a reboot is required was witnessed.
-    This function relies on the salt-minion's ability to create the following
-    volatile registry key in the *HKLM* hive:
+    This function is used to remember that an event indicating that a reboot is
+    required was witnessed. This function relies on the salt-minion's ability to
+    create the following volatile registry key in the *HKLM* hive:
 
        *SYSTEM\\CurrentControlSet\\Services\\salt-minion\\Volatile-Data*
 
-    Because this registry key is volatile, it will not persist
-    beyond the current boot session.
-    Also, in the scope of this key, the name *'Reboot required'* will be
-    assigned the value of *1*.
+    Because this registry key is volatile, it will not persist beyond the
+    current boot session. Also, in the scope of this key, the name *'Reboot
+    required'* will be assigned the value of *1*.
 
-    (For the time being, this this function is being used
-    whenever an install completes with exit code 3010 and
-    this usage can be extended where appropriate in the future.)
+    (For the time being, this this function is being used whenever an install
+    completes with exit code 3010 and this usage can be extended where
+    appropriate in the future.)
 
-    :return: A boolean indicating whether or not the salt minion was
-       able to perform the necessary registry operations.
+    .. versionadded:: 2016.11.0
 
-    :rtype: bool
+    Returns:
+        bool: True if registry entry set successfuly, otherwise False.
 
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' system.set_reboot_required_witnessed
-
     '''
     return __salt__['reg.set_value'](hive='HKLM',
                                      key=MINION_VOLATILE_KEY,
@@ -1249,20 +1330,16 @@ def set_reboot_required_witnessed():
 
 def get_reboot_required_witnessed():
     '''
+    This tells us if, at any time during the current boot session the salt
+    minion witnessed an event indicating that a reboot is required. (For the
+    time being, this function will return True if an install completed with exit
+    code 3010 during the current boot session and this usage can be extended
+    where appropriate in the future)
+
     .. versionadded:: 2016.11.0
 
-    This tells us if, at any time during the current boot session
-    the salt minion witnessed an event indicating
-    that a reboot is required.
-    (For the time being, this function will return True
-    if an install completed with exit code 3010 during the current
-    boot session and this usage can be extended where appropriate
-    in the future)
-
-    :return: a boolean which will be True if the salt-minion reported
-       a required reboot during the current boot session, otherwise False.
-
-    :rtype: bool
+    Returns:
+        bool: True if reboot required, otherwise False.
 
     CLI Example:
 
@@ -1281,11 +1358,11 @@ def get_pending_reboot():
     '''
     Determine whether there is a reboot pending.
 
-    :return: A boolean representing whether reboots are pending.
-    :rtype: bool
-
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if pending reboot, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
