@@ -10,18 +10,19 @@ ext_pillar, simply ported to use mako instead of jinja2 for templating.
 It supports the following features:
 
 - multiple config files that are mako templates with support for ``pillar``,
-  ``__grains__``, ``__salt__``, ``__opts__`` objects
-- a config file renders as an ordered list of files (paths of these files are
-  relative to the current config file)
-- this list of files are read in ordered as mako templates with support for
-  ``stack``, ``pillar``, ``__grains__``, ``__salt__``, ``__opts__`` objects
-- all these rendered files are then parsed as ``yaml``
-- then all yaml dicts are merged in order with support for the following
+  ``__grains__``, ``__salt__``, ``__opts__`` objects.
+- a config file renders as an ordered list of files. Unless absolute, the paths
+  of these files are relative to the current config file - if absolute, they
+  will be treated literally.
+- this list of files are read in order as mako templates with support for
+  ``stack``, ``pillar``, ``__grains__``, ``__salt__``, ``__opts__`` objects.
+- all these rendered files are then parsed as ``yaml``.
+- then all yaml dicts are merged in order, with support for the following.
   merging strategies: ``merge-first``, ``merge-last``, ``remove``, and
-  ``overwrite``
+  ``overwrite``.
 - stack config files can be matched based on ``pillar``, ``grains``, or
   ``opts`` values, which make it possible to support kind of self-contained
-  environments
+  environments.
 
 Configuration in Salt
 ---------------------
@@ -40,7 +41,7 @@ MakoStack config file like below:
 .. code:: yaml
 
     ext_pillar:
-      - stack: /path/to/stack.cfg
+      - makostack: /path/to/stack.cfg
 
 List of config files
 ~~~~~~~~~~~~~~~~~~~~
@@ -50,14 +51,14 @@ You can also provide a list of config files:
 .. code:: yaml
 
     ext_pillar:
-      - stack:
+      - makostack:
           - /path/to/stack1.cfg
           - /path/to/stack2.cfg
 
 Select config files through grains|pillar|opts matching
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-You can also opt for a much more flexible configuration: MakoStack allows to
+You can also opt for a much more flexible configuration: MakoStack allows one to
 select the config files for the current minion based on matching values from
 either grains, or pillar, or opts objects.
 
@@ -66,7 +67,7 @@ Here is an example of such a configuration, which should speak by itself:
 .. code:: yaml
 
     ext_pillar:
-      - stack:
+      - makostack:
           pillar:environment:
             dev: /path/to/dev/stack.cfg
             prod: /path/to/prod/stack.cfg
@@ -77,7 +78,6 @@ Here is an example of such a configuration, which should speak by itself:
           opts:custom:opt:
             value: /path/to/stack0.cfg
 
-
 Grafting data from files to arbitrary namespaces
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
@@ -109,8 +109,10 @@ The config files that are referenced in the above ``ext_pillar`` configuration
 are mako templates which must render as a simple ordered list of ``yaml``
 files that will then be merged to build pillar data.
 
-The path of these ``yaml`` files must be relative to the directory of the
-MakoStack config file.
+Unless an absolute path name is specified, the path of these ``yaml`` files is
+assumed to be relative to the directory containing the MakoStack config file.
+If a path begins with '/', however, it will be treated literally and can be
+anywhere on the filesystem.
 
 The following variables are available in mako templating of makostack
 configuration files:
@@ -428,7 +430,7 @@ def ext_pillar(minion_id, pillar, *args, **kwargs):
         else:
             namespace = None
         if not os.path.isfile(cfg):
-            log.warning('Ignoring pillar stack cfg "{0}": '
+            log.warning('Ignoring Stack cfg "{0}": '
                         'file does not exist'.format(cfg))
             continue
         stack = _process_stack_cfg(cfg, stack, minion_id, pillar, namespace)
@@ -444,6 +446,10 @@ def _process_stack_cfg(cfg, stack, minion_id, pillar, namespace):
                                                 minion_id=minion_id,
                                                 pillar=pillar, stack=stack)
     for path in _parse_top_cfg(tops):
+        dirs = [basedir]
+        if path.startswith('/'):
+            dirs += ['/']
+        lookup = TemplateLookup(directories=dirs)
         try:
             p = lookup.get_template(path).render(__opts__=__opts__,
                                                  __salt__=__salt__,
@@ -452,7 +458,7 @@ def _process_stack_cfg(cfg, stack, minion_id, pillar, namespace):
                                                  pillar=pillar, stack=stack)
             obj = yaml.safe_load(p)
             if not isinstance(obj, dict):
-                log.info('Ignoring pillar stack template "{0}": Can\'t parse '
+                log.info('Ignoring Stack template "{0}": Can\'t parse '
                          'as a valid yaml dictionary'.format(path))
                 continue
             if namespace:
@@ -464,7 +470,7 @@ def _process_stack_cfg(cfg, stack, minion_id, pillar, namespace):
             log.info('Stack template "{0}" not found.'.format(path))
             continue
         except Exception as e:
-            log.info('Ignoring pillar stack template "{0}":'.format(path))
+            log.info('Ignoring Stack template "{0}":'.format(path))
             log.info('{0}'.format(exceptions.text_error_template().render()))
             continue
     return stack
