@@ -22,6 +22,7 @@ from salt.ext.six.moves.urllib.request import urlopen as _urlopen  # pylint: dis
 # Import salt libs
 import salt.key
 import salt.utils
+import salt.utils.compat
 import salt.utils.minions
 import salt.client
 import salt.client.ssh
@@ -669,7 +670,7 @@ def versions():
             ver_diff = -2
         else:
             minion_version = salt.version.SaltStackVersion.parse(minions[minion])
-            ver_diff = cmp(minion_version, master_version)
+            ver_diff = salt.utils.compat.cmp(minion_version, master_version)
 
         if ver_diff not in version_status:
             version_status[ver_diff] = {}
