@@ -144,12 +144,20 @@ import os
 
 # Import salt libs
 import salt.utils
+import salt.utils.files
 from salt.modules.cron import (
     _needs_change,
     _cron_matched
 )
 
 
+def __virtual__():
+    if 'cron.list_tab' in __salt__:
+        return True
+    else:
+        return (False, 'cron module could not be loaded')
+
+
 def _check_cron(user,
                 cmd,
                 minute=None,
@@ -396,7 +404,7 @@ def absent(name,
            'comment': ''}
 
     if __opts__['test']:
-        status = _check_cron(user, name)
+        status = _check_cron(user, name, identifier=identifier)
         ret['result'] = None
         if status == 'absent':
             ret['result'] = True
@@ -517,9 +525,17 @@ def file(name,
     '''
     # Initial set up
     mode = '0600'
-    owner, group, crontab_dir = _get_cron_info()
 
-    cron_path = salt.utils.mkstemp()
+    try:
+        group = __salt__['user.info'](user)['groups'][0]
+    except Exception:
+        ret = {'changes': {},
+               'comment': "Could not identify group for user {0}".format(user),
+               'name': name,
+               'result': False}
+        return ret
+
+    cron_path = salt.utils.files.mkstemp()
     with salt.utils.fopen(cron_path, 'w+') as fp_:
         raw_cron = __salt__['cron.raw_cron'](user)
         if not raw_cron.endswith('\n'):
@@ -546,7 +562,7 @@ def file(name,
                                              source,
                                              source_hash,
                                              source_hash_name,
-                                             owner,
+                                             user,
                                              group,
                                              mode,
                                              template,
@@ -572,7 +588,7 @@ def file(name,
             source,
             source_hash,
             source_hash_name,
-            owner,
+            user,
             group,
             mode,
             __env__,
@@ -600,7 +616,7 @@ def file(name,
             ret,
             source,
             source_sum,
-            owner,
+            user,
             group,
             mode,
             __env__,
@@ -613,17 +629,22 @@ def file(name,
         return ret
 
     cron_ret = None
-    if ret['changes']:
+    if "diff" in ret['changes']:
         cron_ret = __salt__['cron.write_cron_file_verbose'](user, cron_path)
-        ret['comment'] = 'Crontab for user {0} was updated'.format(user)
+        # Check cmd return code and show success or failure
+        if cron_ret['retcode'] == 0:
+            ret['comment'] = 'Crontab for user {0} was updated'.format(user)
+            ret['result'] = True
+            ret['changes'] = ret['changes']['diff']
+        else:
+            ret['comment'] = 'Unable to update user {0} crontab {1}.' \
+                             ' Error: {2}'.format(user, cron_path, cron_ret['stderr'])
+            ret['result'] = False
+            ret['changes'] = {}
     elif ret['result']:
         ret['comment'] = 'Crontab for user {0} is in the correct ' \
                          'state'.format(user)
-
-    if cron_ret and cron_ret['retcode']:
-        ret['comment'] = 'Unable to update user {0} crontab {1}.' \
-                         ' Error: {2}'.format(user, cron_path, cron_ret['stderr'])
-        ret['result'] = False
+        ret['changes'] = {}
 
     os.unlink(cron_path)
     return ret
