@@ -178,6 +178,7 @@ from __future__ import absolute_import
 import salt.loader
 import salt.utils
 import salt.utils.jid
+from salt.ext import six
 from salt.ext.six.moves import range
 from salt.ext.six.moves import zip
 from salt.exceptions import SaltInvocationError
@@ -262,6 +263,7 @@ def run(**kwargs):
     missing = []
     tests = []
     for func in functions:
+        func = func.split(':')[0]
         if func not in __salt__:
             missing.append(func)
         elif __opts__['test']:
@@ -284,8 +286,9 @@ def run(**kwargs):
         failures = []
         success = []
         for func in functions:
+            _func = func.split(':')[0]
             try:
-                func_ret = _call_function(func, returner=kwargs.get('returner'),
+                func_ret = _call_function(_func, returner=kwargs.get('returner'),
                                           func_args=kwargs.get(func))
                 if not _get_result(func_ret, ret['changes'].get('ret', {})):
                     if isinstance(func_ret, dict):
@@ -312,23 +315,45 @@ def _call_function(name, returner=None, **kwargs):
     :return:
     '''
     argspec = salt.utils.args.get_function_argspec(__salt__[name])
-    func_kw = dict(zip(argspec.args[-len(argspec.defaults or []):],  # pylint: disable=incompatible-py3-code
-                       argspec.defaults or []))
-    func_args = []
-    for funcset in kwargs.get('func_args') or {}:
-        if isinstance(funcset, dict):
-            func_kw.update(funcset)
-        else:
-            func_args.append(funcset)
 
+    # func_kw is initialized to a dictionary of keyword arguments the function to be run accepts
+    func_kw = dict(zip(argspec.args[-len(argspec.defaults or []):],  # pylint: disable=incompatible-py3-code
+                   argspec.defaults or []))
+
+    # func_args is initialized to a list of positional arguments that the function to be run accepts
+    func_args = argspec.args[:len(argspec.args or []) - len(argspec.defaults or [])]
+    arg_type, na_type, kw_type = [], {}, False
+    for funcset in reversed(kwargs.get('func_args') or []):
+        if not isinstance(funcset, dict):
+            # We are just receiving a list of args to the function to be run, so just append
+            # those to the arg list that we will pass to the func.
+            arg_type.append(funcset)
+        else:
+            for kwarg_key in six.iterkeys(funcset):
+                # We are going to pass in a keyword argument. The trick here is to make certain
+                # that if we find that in the *args* list that we pass it there and not as a kwarg
+                if kwarg_key in func_args:
+                    arg_type.append(funcset[kwarg_key])
+                    continue
+                else:
+                    # Otherwise, we're good and just go ahead and pass the keyword/value pair into
+                    # the kwargs list to be run.
+                    func_kw.update(funcset)
+    arg_type.reverse()
+    _exp_prm = len(argspec.args or []) - len(argspec.defaults or [])
+    _passed_prm = len(arg_type)
     missing = []
-    for arg in argspec.args:
-        if arg not in func_kw:
-            missing.append(arg)
+    if na_type and _exp_prm > _passed_prm:
+        for arg in argspec.args:
+            if arg not in func_kw:
+                missing.append(arg)
     if missing:
         raise SaltInvocationError('Missing arguments: {0}'.format(', '.join(missing)))
+    elif _exp_prm > _passed_prm:
+        raise SaltInvocationError('Function expects {0} parameters, got only {1}'.format(
+            _exp_prm, _passed_prm))
 
-    mret = __salt__[name](*func_args, **func_kw)
+    mret = __salt__[name](*arg_type, **func_kw)
     if returner is not None:
         returners = salt.loader.returners(__opts__, __salt__)
         if returner in returners:
