@@ -75,7 +75,7 @@ class AESReqServerMixin(object):
             # cases, 'aes' is already set in the secrets.
             salt.master.SMaster.secrets['aes'] = {
                 'secret': multiprocessing.Array(ctypes.c_char,
-                              salt.crypt.Crypticle.generate_key_string()),
+                              six.b(salt.crypt.Crypticle.generate_key_string())),
                 'reload': salt.crypt.Crypticle.generate_key_string
             }
 
@@ -117,7 +117,7 @@ class AESReqServerMixin(object):
             return self.crypticle.dumps({})
         except IOError:
             log.error('AES key not found')
-            return 'AES key not found'
+            return {'error': 'AES key not found'}
 
         pret = {}
         cipher = PKCS1_OAEP.new(pub)
@@ -250,6 +250,7 @@ class AESReqServerMixin(object):
                         fp_.write(load['pub'])
                     eload = {'result': False,
                              'id': load['id'],
+                             'act': 'denied',
                              'pub': load['pub']}
                     self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
                     return {'enc': 'clear',
@@ -338,6 +339,7 @@ class AESReqServerMixin(object):
                             fp_.write(load['pub'])
                         eload = {'result': False,
                                  'id': load['id'],
+                                 'act': 'denied',
                                  'pub': load['pub']}
                         self.event.fire_event(eload, salt.utils.event.tagify(prefix='auth'))
                         return {'enc': 'clear',
