@@ -466,8 +466,8 @@ def query(url,
         # We want to use curl_http if we have a proxy defined
         if proxy_host and proxy_port:
             if HAS_CURL_HTTPCLIENT is False:
-                ret['error'] = ('proxy_host and proxy_port has been set. This requires pycurl, but the '
-                                'pycurl library does not seem to be installed')
+                ret['error'] = ('proxy_host and proxy_port has been set. This requires pycurl and tornado, '
+                                'but the libraries does not seem to be installed')
                 log.error(ret['error'])
                 return ret
 
@@ -779,7 +779,8 @@ def _render(template, render, renderer, template_dict, opts):
         blacklist = opts.get('renderer_blacklist')
         whitelist = opts.get('renderer_whitelist')
         ret = compile_template(template, rend, renderer, blacklist, whitelist, **template_dict)
-        ret = ret.read()
+        if salt.utils.stringio.is_readable(ret):
+            ret = ret.read()
         if str(ret).startswith('#!') and not str(ret).startswith('#!/'):
             ret = str(ret).split('\n', 1)[1]
         return ret
