@@ -15,6 +15,7 @@ import contextlib
 import subprocess
 import multiprocessing
 import multiprocessing.util
+import socket
 
 
 # Import salt libs
@@ -55,7 +56,20 @@ def notify_systemd():
         import systemd.daemon
     except ImportError:
         if salt.utils.which('systemd-notify') and systemd_notify_call('--booted'):
-            return systemd_notify_call('--ready')
+            # Notify systemd synchronously
+            notify_socket = os.getenv('NOTIFY_SOCKET')
+            if notify_socket:
+                # Handle abstract namespace socket
+                if notify_socket.startswith('@'):
+                    notify_socket = '\0{0}'.format(notify_socket[1:])
+                try:
+                    sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
+                    sock.connect(notify_socket)
+                    sock.sendall('READY=1'.encode())
+                    sock.close()
+                except socket.error:
+                    return systemd_notify_call('--ready')
+                return True
         return False
 
     if systemd.daemon.booted():
@@ -130,8 +144,10 @@ def get_pidfile(pidfile):
     '''
     with salt.utils.fopen(pidfile) as pdf:
         pid = pdf.read()
-
-    return int(pid)
+    if pid:
+        return int(pid)
+    else:
+        return
 
 
 def clean_proc(proc, wait_for_kill=10):
@@ -278,7 +294,7 @@ class ProcessManager(object):
             kwargs = {}
 
         if salt.utils.is_windows():
-            # Need to ensure that 'log_queue' is correctly transfered to
+            # Need to ensure that 'log_queue' is correctly transferred to
             # processes that inherit from 'MultiprocessingProcess'.
             if type(MultiprocessingProcess) is type(tgt) and (
                     issubclass(tgt, MultiprocessingProcess)):
@@ -399,6 +415,8 @@ class ProcessManager(object):
                     yield gen.sleep(10)
                 else:
                     time.sleep(10)
+                if len(self._process_map) == 0:
+                    break
             # OSError is raised if a signal handler is called (SIGTERM) during os.wait
             except OSError:
                 break
@@ -416,6 +434,7 @@ class ProcessManager(object):
         if self._restart_processes is True:
             for pid, mapping in six.iteritems(self._process_map):
                 if not mapping['Process'].is_alive():
+                    log.trace('Process restart of {0}'.format(pid))
                     self.restart_process(pid)
 
     def kill_children(self, *args, **kwargs):
@@ -685,6 +704,12 @@ class SignalHandlingMultiprocessingProcess(MultiprocessingProcess):
             msg += 'SIGTERM'
         msg += '. Exiting'
         log.debug(msg)
+        if HAS_PSUTIL:
+            process = psutil.Process(self.pid)
+            if hasattr(process, 'children'):
+                for child in process.children(recursive=True):
+                    if child.is_running():
+                        child.terminate()
         sys.exit(salt.defaults.exitcodes.EX_OK)
 
     def start(self):
@@ -697,8 +722,8 @@ def default_signals(*signals):
     old_signals = {}
     for signum in signals:
         try:
-            signal.signal(signum, signal.SIG_DFL)
             old_signals[signum] = signal.getsignal(signum)
+            signal.signal(signum, signal.SIG_DFL)
         except ValueError as exc:
             # This happens when a netapi module attempts to run a function
             # using wheel_async, because the process trying to register signals
